@@ -14,6 +14,8 @@ sub suite {
 	my $class = shift;
 	my $suite = Test::Unit::TestSuite->empty_new("LXR Tests");
 	$suite->add_test(Test::Unit::TestSuite->new("ConfigTest"));
+	$suite->add_test(Test::Unit::TestSuite->new("SecurityTest"));
+	$suite->add_test(Test::Unit::TestSuite->new("CVSTest"));
 
 	return $suite;
 }
